﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Net;
using System.IO;
using Newtonsoft.Json;
using APIExample.Models;
using System.Threading;

namespace APIExample
{
    class Program
    {
        const string authCompanyId = "-1";                                  //use your company ID here
        const string authApiKey = "99ace875-2d2e-550a-bd9a-d16330d24fea";   //use your API key here

        static void Main(string[] args)
        {
            long projectId = -1;

            #region Get supported languages

            Console.WriteLine("Get a collection of supported languages");
            Console.WriteLine("---------------------------------------------------------------------------------");
            string uriLanguages = "https://www.textunited.com/api/languages";
            Console.WriteLine("\nURI: {0}", uriLanguages);

            WebRequest reqLanguages = GetRequest(uriLanguages, "GET");
            WebResponse resLanguages = reqLanguages.GetResponse();
            string jsonLanguages = GetResponseJson(resLanguages);

            Console.WriteLine("\nRaw JSON:");
            Console.WriteLine(jsonLanguages);
            IEnumerable<ApiSystemLanguage> languages = JsonConvert.DeserializeObject<IEnumerable<ApiSystemLanguage>>(jsonLanguages);

            Console.WriteLine("\nCollection of language objects (10 elements from the begining of the list):");
            foreach (ApiSystemLanguage language in languages.Take(10))
                Console.WriteLine("{0}\t{1}", language.Id, language.LangCode);

            Console.WriteLine("\nPress any key to continue...");
            Console.ReadKey();

            #endregion

            #region Get company users

            Console.WriteLine("\n\nGet a collection of comapny users");
            Console.WriteLine("---------------------------------------------------------------------------------");
            string uriUsers = "https://www.textunited.com/api/employees";
            Console.WriteLine("\nURI: {0}", uriUsers);

            WebRequest reqUsers = GetRequest(uriUsers, "GET");
            WebResponse resUsers = reqUsers.GetResponse();
            string jsonUsers = GetResponseJson(resUsers);

            Console.WriteLine("\nRaw JSON:");
            Console.WriteLine(jsonUsers);
            IEnumerable<ApiUser> users = JsonConvert.DeserializeObject<IEnumerable<ApiUser>>(jsonUsers);

            Console.WriteLine("\nCollection of user objects:");
            foreach (ApiUser user in users)
                Console.WriteLine("{0}\t{1} {2} ({3})", user.Id, user.FirstName, user.LastName, user.Email);

            Console.WriteLine("\nPress any key to continue...");
            Console.ReadKey();

            #endregion

            #region Create a project with one resx file for translation

            byte[] sourceFileContent = File.ReadAllBytes("SampleResourceFile.resx");
            string sourceFileBase64 = Convert.ToBase64String(sourceFileContent);
            List<ApiNewFile> projectSourceFiles = new List<ApiNewFile>();
            projectSourceFiles.Add(new ApiNewFile() { Filename = "SampleResourceFile.resx", Content = sourceFileBase64 });

            ApiSystemLanguage langEnUs = languages.First(l => l.LangCode == "EN-US");
            ApiSystemLanguage langDeAt = languages.First(l => l.LangCode == "DE-AT");
            ApiUser translator = users.First();

            ApiNewProject newProject = new ApiNewProject()
            {
                ProjectName = "Testing API - 1",
                SourceLanguageId = langEnUs.Id,
                TargetLanguageId = langDeAt.Id,
                TranslatorId = translator.Id,

                Description = "This is a sample project for testing API methods",
                EndDate = DateTime.Now.AddDays(7),

                Files = projectSourceFiles
            };

            Console.WriteLine("\n\nCreate new project");
            Console.WriteLine("---------------------------------------------------------------------------------");
            string uriNewProject = "https://www.textunited.com/api/fastproject";
            Console.WriteLine("\nURI: {0}", uriNewProject);

            string jsonNewProject;
            WebRequest reqNewProject = GetPostRequest(uriNewProject, newProject, out jsonNewProject);
            Console.WriteLine("\nRaw JSON of the project to be created:");
            Console.WriteLine(jsonNewProject);

            WebResponse resNewProject = reqNewProject.GetResponse();
            string jsonProjectId = GetResponseJson(resNewProject);


            Console.WriteLine("\nRaw JSON in response (ID of the newly created project):");
            Console.WriteLine(jsonProjectId);

            string sProjectId = JsonConvert.DeserializeObject<string>(jsonProjectId);
            projectId = long.Parse(sProjectId);

            Console.WriteLine("\nPress any key to continue...");
            Console.ReadKey();

            #endregion

            #region Get a collection of projects to see if it's ready for translation

            while (true)
            {
                Console.WriteLine("\n\nGet a collection of projects to see if it's ready for translation");
                Console.WriteLine("---------------------------------------------------------------------------------");
                string uriProjects = "https://www.textunited.com/api/projects";
                Console.WriteLine("\nURI: {0}", uriProjects);

                WebRequest reqProjects = GetRequest(uriProjects, "GET");
                WebResponse resProjects = reqProjects.GetResponse();
                string jsonProjects = GetResponseJson(resProjects);

                Console.WriteLine("\nRaw JSON:");
                Console.WriteLine(jsonProjects);
                IEnumerable<ApiProject> projects = JsonConvert.DeserializeObject<IEnumerable<ApiProject>>(jsonProjects);

                Console.WriteLine("\nCollection of project objects:");
                foreach (ApiProject project in projects)
                    Console.WriteLine("{0}\t{1}, {2}->{3}, {4}", project.Id, project.Name, project.SourceLanguageCode, project.TargetLanguageCode, project.State);

                if (projects.Any(p => (p.Id == projectId) && (p.State == "In progress")))
                    break;

                Thread.Sleep(5000);
            }
            Console.WriteLine("\nPress any key to continue...");
            Console.ReadKey();

            #endregion


            #region Read project details

            Console.WriteLine("\n\nGet project details");
            Console.WriteLine("---------------------------------------------------------------------------------");
            string uriProjectDetails = string.Format("https://www.textunited.com/api/projects/{0}", projectId);
            Console.WriteLine("\nURI: {0}", uriProjectDetails);

            WebRequest reqProjectDetails = GetRequest(uriProjectDetails, "GET");
            WebResponse resProjectDetails = reqProjectDetails.GetResponse();
            string jsonProjectDetails = GetResponseJson(resProjectDetails);

            Console.WriteLine("\nRaw JSON:");
            Console.WriteLine(jsonProjectDetails);
            ApiProjectDetails projectDetails = JsonConvert.DeserializeObject<ApiProjectDetails>(jsonProjectDetails);

            Console.WriteLine();
            Console.WriteLine("Project ID: {0}", projectDetails.Id);
            Console.WriteLine("Status: {0}", projectDetails.State);
            Console.WriteLine("Progress: {0}", projectDetails.Progress);

            Console.WriteLine("\nPress any key to continue...");
            Console.ReadKey();

            #endregion

            #region Read project files

            Console.WriteLine("\n\nGet a collection of project files");
            Console.WriteLine("---------------------------------------------------------------------------------");
            string uriProjectFiles = string.Format("https://www.textunited.com/api/projectfiles?projectId={0}", projectId);
            Console.WriteLine("\nURI: {0}", uriProjectFiles);

            WebRequest reqProjectFiles = GetRequest(uriProjectFiles, "GET");
            WebResponse resProjectFiles = reqProjectFiles.GetResponse();
            string jsonProjectFiles = GetResponseJson(resProjectFiles);

            Console.WriteLine("\nRaw JSON:");
            Console.WriteLine(jsonProjectFiles);
            IEnumerable<ApiProjectFile> files = JsonConvert.DeserializeObject<IEnumerable<ApiProjectFile>>(jsonProjectFiles);

            Console.WriteLine("\nCollection of project file objects:");
            foreach (ApiProjectFile file in files)
                Console.WriteLine("{0}\t{1}\t{2}B\t{3}", file.FileId, file.Filename, file.FileSize, file.Status);

            Console.WriteLine("\nPress any key to continue...");
            Console.ReadKey();

            #endregion

            Console.WriteLine();
            Console.WriteLine("##################################################################################");
            Console.WriteLine("NOW YOU MAY TRANSLATE SOME OR ALL SEGMENTS IN THE TEXT UNITED APP OR IN THE PORTAL");
            Console.WriteLine("##################################################################################");
            Console.WriteLine("\nPress any key to continue...");
            Console.ReadKey();

            #region Read project details to see its progress

            reqProjectDetails = GetRequest(uriProjectDetails, "GET");
            resProjectDetails = reqProjectDetails.GetResponse();
            jsonProjectDetails = GetResponseJson(resProjectDetails);

            Console.WriteLine("\nRaw JSON:");
            Console.WriteLine(jsonProjectDetails);
            projectDetails = JsonConvert.DeserializeObject<ApiProjectDetails>(jsonProjectDetails);

            Console.WriteLine();
            Console.WriteLine("Project ID: {0}", projectDetails.Id);
            Console.WriteLine("Status: {0}", projectDetails.State);
            Console.WriteLine("Progress: {0}", projectDetails.Progress);

            Console.WriteLine("\nPress any key to continue...");
            Console.ReadKey();

            #endregion

            #region Request generating translated versions of the files in the project

            Console.WriteLine("\n\nRequest generating translated versions of the files");
            Console.WriteLine("---------------------------------------------------------------------------------");
            string uriGenerateTargetFiles = string.Format("https://www.textunited.com/api/projectfiles?projectId={0}", projectId);
            Console.WriteLine("\nURI: {0}", uriGenerateTargetFiles);

            WebRequest reqGenerateTargetFiles = GetRequest(uriGenerateTargetFiles, "PUT");
            reqGenerateTargetFiles.ContentLength = 0;
            WebResponse resGenerateTargetFiles = reqGenerateTargetFiles.GetResponse();
            string jsonGenerateTargetFiles = GetResponseJson(resGenerateTargetFiles);

            Console.WriteLine("\nRaw JSON:");
            Console.WriteLine(jsonGenerateTargetFiles);
            bool bGenerateTargetFiles = JsonConvert.DeserializeObject<bool>(jsonGenerateTargetFiles);

            Console.WriteLine();
            Console.WriteLine("Successfully requested: {0}", bGenerateTargetFiles);

            //Console.WriteLine("\nPress any key to continue...");
            //Console.ReadKey();

            #endregion

            #region Wait for the target file to be generated

            Console.WriteLine("\n\nWaiting for the target file to be generated");
            Console.WriteLine("---------------------------------------------------------------------------------");
            while (true)
            {
                Console.Write("Checking...  ");
                reqProjectFiles = GetRequest(uriProjectFiles, "GET");
                resProjectFiles = reqProjectFiles.GetResponse();
                jsonProjectFiles = GetResponseJson(resProjectFiles);
                files = JsonConvert.DeserializeObject<IEnumerable<ApiProjectFile>>(jsonProjectFiles);
                Console.WriteLine("{0}\t{1}\t{2}B\t{3}", files.First().FileId, files.First().Filename, files.First().FileSize, files.First().Status);
                if (files.First().Status == "Translated") break;
                Thread.Sleep(1000);
            }

            Console.WriteLine("\n\nThe translated version of the file has been generated. You may download it now.");
            Console.WriteLine("\nPress any key to continue...");
            Console.ReadKey();


            #endregion

            #region Download the translated version of the file

            long fileId = files.First().FileId;

            Console.WriteLine("\n\nDownload project file");
            Console.WriteLine("---------------------------------------------------------------------------------");
            string uriDownload = string.Format("https://www.textunited.com/api/projectfiles?projectId={0}&fileId={1}&type=translated", projectId, fileId);
            Console.WriteLine("\nURI: {0}", uriProjectFiles);

            WebRequest reqDownload = GetRequest(uriDownload, "GET");
            WebResponse resDownload = reqDownload.GetResponse();
            string jsonDownload = GetResponseJson(resDownload);

            Console.WriteLine("\nRaw JSON:");
            Console.WriteLine(jsonDownload);
            ApiProjectFileContent downloadedFile = JsonConvert.DeserializeObject<ApiProjectFileContent>(jsonDownload);

            byte[] byteContent = Convert.FromBase64String(downloadedFile.Content);
            File.WriteAllBytes("translated." + downloadedFile.Filename, byteContent);

            Console.WriteLine("\nPress any key to continue...");
            Console.ReadKey();

            #endregion

            #region Update the project with a new file

            byte[] sourceUpdateFileContent = File.ReadAllBytes("SampleResourceUpdateFile.resx");
            string sourceUpdateFileBase64 = Convert.ToBase64String(sourceUpdateFileContent);
            List<ApiNewFile> projectUpdateSourceFiles = new List<ApiNewFile>();
            projectUpdateSourceFiles.Add(new ApiNewFile() { Filename = "SampleResourceUpdateFile.resx", Content = sourceUpdateFileBase64 });

            ApiUpdateProject updateProject = new ApiUpdateProject()
            {
                ProjectId = projectId,
                Files = projectUpdateSourceFiles
            };

            Console.WriteLine("\n\nUpdate project");
            Console.WriteLine("---------------------------------------------------------------------------------");
            string uriUpdateProject = "https://www.textunited.com/api/projectfiles";
            Console.WriteLine("\nURI: {0}", uriUpdateProject);

            string jsonUpdateProject;
            WebRequest reqUpdateProject = GetPostRequest(uriUpdateProject, updateProject, out jsonUpdateProject);
            Console.WriteLine("\nRaw JSON of the project to be updated:");
            Console.WriteLine(jsonUpdateProject);

            WebResponse resUpdateProject = reqUpdateProject.GetResponse();

            Console.WriteLine("\nRaw JSON in response:");
            Console.WriteLine(GetResponseJson(resUpdateProject));

            Console.WriteLine("\nPress any key to continue...");
            Console.ReadKey();


            #endregion
        }

        static WebRequest GetRequest(string uri, string httpMethod)
        {
            WebRequest request = (HttpWebRequest)WebRequest.Create(uri);
            string auth = string.Format("{0}:{1}", authCompanyId, authApiKey);
            request.Headers.Add(HttpRequestHeader.Authorization, "Basic " + Convert.ToBase64String(System.Text.ASCIIEncoding.ASCII.GetBytes(auth)));
            request.Method = httpMethod;

            return request;
        }

        static WebRequest GetPostRequest(string uri, object modelToPost, out string jsonToPost)
        {
            WebRequest request = GetRequest(uri, "POST");
            request.ContentType = "application/json; charset=utf-8";

            using (var streamWriter = new StreamWriter(request.GetRequestStream()))
            {
                jsonToPost = JsonConvert.SerializeObject(modelToPost);
                streamWriter.Write(jsonToPost);
                streamWriter.Flush();
            }
            return request;
        }

        static string GetResponseJson(WebResponse response)
        {
            Stream dataStream = response.GetResponseStream();
            StreamReader reader = new StreamReader(dataStream);
            string json = reader.ReadToEnd();

            

            return json;
        }
    }
}
